Unit USTreeNodes;

Interface

Uses
  Dialogs, Classes, Graphics, ExtCtrls, USortable, USTreeNode;

Type
  TSTreeNodes = Class(TObject)
  Private
    ICanvas       : TCanvas;
    ICanvasWidth  : Word;
    ICanvasHeight : Word;

    IRoot    : TSTreeNode;
    ICurrent : TSTreeNode;

    Function ReadCanvas       : TCanvas;
    Function ReadCanvasWidth  : Word;
    Function ReadCanvasHeight : Word;

    Procedure WriteCanvas(PCanvas : TCanvas);
    Procedure WriteCanvasWidth(PWidth : Word);
    Procedure WriteCanvasHeight(PHeight : Word);

    Function ReadRoot    : TSTreeNode;
    Function ReadCurrent : TSTreeNode;
  Public
    Constructor Create;

    Function Delete(PSortable : TSortable) : Boolean;
    Function Search(PSortable : TSortable) : TSTreeNode;

    Procedure Left;
    Procedure Right;
    Procedure Reset;
    Procedure Insert(PSortable : TSortable);

    Procedure Draw(PNode : TSTreeNode; ShowBalance : Boolean = False; Redraw : Boolean = True; PDepth : Integer = 0; PXPos : Integer = -1; DifX : Integer = -1);

    Procedure DestroyNodes;

    Property Canvas       : TCanvas Read ReadCanvas       Write WriteCanvas;
    Property CanvasWidth  : Word    Read ReadCanvasWidth  Write WriteCanvasWidth;
    Property CanvasHeight : Word    Read ReadCanvasHeight Write WriteCanvasHeight;

    Property Root    : TSTreeNode Read ReadRoot;
    Property Current : TSTreeNode Read ReadCurrent;
  End;

Implementation


////////////////////////////////////////////////////////////////////////////////
//  Constructor TSTreeNodes.Create;                                           //
//                                                                            //
//  Erstellt die Wurzel und weist CanvasWidth (bentigt fr Draw())           //
//  den Wert 0 (Null) zu.                                                     //
////////////////////////////////////////////////////////////////////////////////

Constructor TSTreeNodes.Create;
Begin
  IRoot        := TSTreeNode.Create;
  ICanvasWidth := 0;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TSTreeNodes.DestroyNodes;                                       //
//                                                                            //
//  Lscht die Knoten, die Root untergerstellt sind (also alle) aus dem RAM.  //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNodes.DestroyNodes;
Begin
  If Assigned(IRoot) Then
    IRoot.Remove;
End;


Function TSTreeNodes.ReadCanvas;
Begin
  Result := ICanvas;
End;


Function TSTreeNodes.ReadCanvasWidth;
Begin
  Result := ICanvasWidth;
End;


Function TSTreeNodes.ReadCanvasHeight;
Begin
  Result := ICanvasHeight;
End;


Function TSTreeNodes.ReadRoot;
Begin
  Result := IRoot;
End;


Function TSTreeNodes.ReadCurrent;
Begin
  Result := ICurrent;
End;


Procedure TSTreeNodes.WriteCanvas;
Begin
  ICanvas := PCanvas;
End;


Procedure TSTreeNodes.WriteCanvasWidth;
Begin
  ICanvasWidth := PWidth;
End;


Procedure TSTreeNodes.WriteCanvasHeight;
Begin
  ICanvasHeight := PHeight;
End;


Procedure TSTreeNodes.Reset;
Begin
  ICurrent := IRoot;
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TSTreeNodes.Insert(PSortable : TSortable);                      //
//                                                                            //
//  Gibt Root den Auftrag,  PSortable einzufgen und zeichnet den Baum neu.   //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNodes.Insert;
Begin
  Root.Insert(PSortable);

  Draw(Root);
End;


////////////////////////////////////////////////////////////////////////////////
//  Function TSTreeNodes.Delete(PSortable : TSortable) : Boolean;             //
//                                                                            //
//  Gibt Root den Auftrag, den Knoten mit PSortable als Inhalt zu lschen     //
//  und aktualisiert danach die Zeichnung.                                    //
//  Es wird zurckgegeben, ob der Knoten gelscht werden konnte.              //
////////////////////////////////////////////////////////////////////////////////

Function TSTreeNodes.Delete;
Begin
  Result := Root.Delete(PSortable);

  Draw(Root);
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TSTreeNodes.Left;                                               //
//                                                                            //
//  Bewegt den Zeiger Current auf den Knoten links von ihm,                   //
//  sofern dieser nicht leer ist.                                             //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNodes.Left;
Begin
  If Not Current.IsLeaf Then
    ICurrent := TSTreeNode(Current.LeftNode);
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TSTreeNodes.Right;                                              //
//                                                                            //
//  Bewegt den Zeiger Current auf den Knoten rechts von ihm,                  //
//  sofern dieser nicht leer ist.                                             //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNodes.Right;
Begin
  If Not Current.IsLeaf Then
    ICurrent := TSTreeNode(Current.RightNode);
End;


////////////////////////////////////////////////////////////////////////////////
//  Procedure TSTreeNodes.Draw(PNode       : TSTreeNode;                      //
//                             ShowBalance : Boolean = False;                 //
//                             Redraw      : Boolean = True;                  //
//                             PDepth      : Integer = 0;                     //
//                             PXPos       : Integer = -1;                    //
//                             DifX        : Integer = -1);                   //
//                                                                            //
//  Zeichnet die Baumstruktur auf Canvas, falls dieser sowie CanvasWidth,     //
//  gesetzt ist. Die letzten drei Parameter sind optional, um den Aufruf      //
//  zu vereinfachen.                                                          //
////////////////////////////////////////////////////////////////////////////////

Procedure TSTreeNodes.Draw;
Var
  XPos,
  YPos,
  Depth    : Integer;
  OldColor : TColor;

Begin
  If ((CanvasWidth > 0) And (CanvasHeight > 0)) Then Begin
    If (Canvas <> Nil) Then Begin
      If (PNode <> Nil) Then Begin
        If (Not PNode.IsEmpty) Then Begin
          If Redraw Then Begin
            OldColor := Canvas.Brush.Color;

            Canvas.Brush.Color := clBtnFace;
            Canvas.FillRect(Rect(0, 0, CanvasWidth, CanvasHeight));

            Canvas.Brush.Color := OldColor;
          End;

          If Difx = -1 Then
            DifX := CanvasWidth Div 2;

          DifX := DifX Div 2;

          Depth := PDepth;

          If PXPos = -1 Then
            XPos := CanvasWidth Div 2
          Else
            XPos := PXPos;

          YPos := Depth * 30 + 20;

          If Not PNode.LeftNode.IsEmpty Then Begin
            Canvas.MoveTo(XPos + 7, YPos);
            Canvas.LineTo(XPos + 7 - DifX, YPos + 30);
          End;

          Draw(TSTreeNode(PNode.LeftNode), ShowBalance, False, Depth + 1, XPos - DifX, DifX);

          If Not PNode.RightNode.IsEmpty Then Begin
            Canvas.MoveTo(XPos, YPos);
            Canvas.LineTo(XPos + DifX, YPos + 30);
          End;

          OldColor := Canvas.Brush.Color;

          If ShowBalance Then Begin
            If PNode.IsBalanced Then
              Canvas.Brush.Color := clGreen
            Else
              Canvas.Brush.Color := clRed;
          End;

          Canvas.Ellipse(XPos - 10, YPos - 5, XPos + 25, YPos + 15);

          Canvas.TextOut(XPos, YPos - 3, PNode.Value.Text);

          If ShowBalance Then
            Canvas.Brush.Color := OldColor;

          Draw(TSTreeNode(PNode.RightNode), ShowBalance, False, Depth + 1, XPos + DifX, DifX);
        End
        Else If Redraw Then Begin
          OldColor := Canvas.Brush.Color;

          Canvas.Brush.Color := clBtnFace;
          Canvas.FillRect(Rect(0, 0, CanvasWidth, CanvasHeight));

          Canvas.Brush.Color := OldColor;
        End;
      End;
    End;
  End;
End;


Function TSTreeNodes.Search;
Begin
  Result := Root.Search(PSortable);
End;


End.
